@@ -140,20 +140,6 @@ anjuta_property_data_type_get_type (void)
   }
   return etype;
 }
-GType
-anjuta_preferences_filter_type_get_type (void)
-{
-  static GType etype = 0;
-  if (etype == 0) {
-    static const GEnumValue values[] = {
-      { ANJUTA_PREFERENCES_FILTER_NONE, "ANJUTA_PREFERENCES_FILTER_NONE", "none" },
-      { ANJUTA_PREFERENCES_FILTER_PROJECT, "ANJUTA_PREFERENCES_FILTER_PROJECT", "project" },
-      { 0, NULL, NULL }
-    };
-    etype = g_enum_register_static (g_intern_static_string ("AnjutaPreferencesFilterType"), values);
-  }
-  return etype;
-}
 
 /* enumerations from "anjuta-launcher.h" */
 GType
