@@ -64,10 +64,9 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <string.h>
+#include <stdlib.h>
 
-#include <glade/glade-parser.h>
 #include <gconf/gconf-client.h>
-#include <libgnomevfs/gnome-vfs.h>
 
 #include <libanjuta/anjuta-preferences.h>
 #include <libanjuta/anjuta-utils.h>
@@ -75,6 +74,9 @@
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/interfaces/ianjuta-preferences.h>
 
+/* AnjutaPreferences is a singleton */
+static AnjutaPreferences* default_preferences = NULL;
+
 struct _AnjutaProperty
 {
 	GtkWidget                *object;
@@ -104,17 +106,20 @@ struct _AnjutaPreferencesPriv
 	GtkWidget           *prefs_dialog;
 	AnjutaPluginManager *plugin_manager;
 	gboolean             is_showing;
+
+	GHashTable			*notifications;
 };
 
 /* Internal structure for anjuta_preferences_foreach */
 struct _AnjutaPreferencesForeachData
 {
 	AnjutaPreferences *pr;
-	AnjutaPreferencesFilterType filter;
 	AnjutaPreferencesCallback callback;
 	gpointer callback_data;
 };
 
+/* Data for notifications */
+
 #define PREFERENCE_PROPERTY_PREFIX "preferences_"
 #define GCONF_KEY_PREFIX "/apps/anjuta/preferences"
 
@@ -126,6 +131,14 @@ build_key (const gchar *key)
 	return buffer;
 }
 
+static const gchar*
+unbuild_key (const gchar* key)
+{
+	if (g_str_has_prefix (key, GCONF_KEY_PREFIX))
+		return key + strlen (GCONF_KEY_PREFIX);
+	return NULL;
+}
+
 /**
  * anjuta_preferences_get:
  * @pr: A #AnjutaPreferences object
@@ -148,58 +161,6 @@ anjuta_preferences_get (AnjutaPreferences *pr, const gchar *key)
 	return gconf_client_get_string (pr->priv->gclient, build_key (key), NULL);
 }
 
-/**
- * anjuta_preferences_get_list:
- * @pr: A #AnjutaPreferences object
- * @key: Property key
- * @list_type: Type of each list element
- *
- * Gets the list of @key.
- *
- * Return value: Key list or NULL if the key is not defined.
- */
-#ifdef __GNUC__
-inline
-#endif
-GSList *
-anjuta_preferences_get_list (AnjutaPreferences *pr, const gchar *key,
-                             GConfValueType list_type)
-{
-	g_return_val_if_fail (ANJUTA_IS_PREFERENCES (pr), NULL);
-	g_return_val_if_fail (key != NULL, NULL);
-
-	return gconf_client_get_list(pr->priv->gclient, build_key (key), list_type, NULL);
-}
-
-/**
- * anjuta_preferences_get_pair:
- * @pr: A #AnjutaPreferences object
- * @key: Property key
- * @car_type: Desired type of the pair's first field (car).
- * @cdr_type: Desired type of the pair's second field (cdr).
- * @car_retloc: Address of a return location for the car.
- * @cdr_retloc: Address of a return location for the cdr.
- *
- * Gets the pair of @key.
- *
- * Return value: TRUE or FALSE.
- */
-#ifdef __GNUC__
-inline
-#endif
-gboolean
-anjuta_preferences_get_pair (AnjutaPreferences *pr, const gchar *key,
-                             GConfValueType car_type, GConfValueType cdr_type,
-                             gpointer car_retloc, gpointer cdr_retloc)
-{
-	g_return_val_if_fail (ANJUTA_IS_PREFERENCES (pr), FALSE);
-	g_return_val_if_fail (key != NULL, FALSE);
-
-	return gconf_client_get_pair(pr->priv->gclient, build_key (key),
-                                 car_type, cdr_type,
-                                 car_retloc, cdr_retloc, NULL);
-}
-
 /**
  * anjuta_preferences_get_int:
  * @pr: A #AnjutaPreferences object
@@ -230,16 +191,55 @@ anjuta_preferences_get_int (AnjutaPreferences *pr, const gchar *key)
 			case GCONF_VALUE_INT:
 				ret_val = gconf_value_get_int (value);
 				break;
-			case GCONF_VALUE_BOOL:
-				ret_val = gconf_value_get_bool (value);
-				break;
 			default:
 				g_warning ("Invalid gconf type for key: %s", key);
 		}
 		gconf_value_free (value);
 	}
-	/* else
-		g_warning ("The preference key %s is not defined", key); */
+	else
+		DEBUG_PRINT ("The preference key %s is not defined", key);
+	return ret_val;
+}
+
+/**
+ * anjuta_preferences_get_int:
+ * @pr: A #AnjutaPreferences object
+ * @key: Property key
+ *
+ * Gets the value of @key as integer.
+ *
+ * Return value: Key value as boolean or FALSE if the key is not defined.
+ */
+#ifdef __GNUC__
+inline
+#endif
+gint
+anjuta_preferences_get_bool (AnjutaPreferences *pr, const gchar *key)
+{
+	gboolean ret_val;
+	GConfValue *value;
+	
+	g_return_val_if_fail (ANJUTA_IS_PREFERENCES (pr), 0);
+	g_return_val_if_fail (key != NULL, 0);
+	
+	ret_val = FALSE;
+	value = gconf_client_get (pr->priv->gclient, build_key (key), NULL);
+	if (value)
+	{
+		switch (value->type)
+		{
+			case GCONF_VALUE_BOOL:
+				ret_val = gconf_value_get_bool (value);
+				break;
+			case GCONF_VALUE_INT:
+				ret_val = (gboolean) gconf_value_get_int (value);
+			default:
+				g_warning ("Invalid gconf type for key: %s", key);
+		}
+		gconf_value_free (value);
+	}
+	else
+		DEBUG_PRINT ("The preference key %s is not defined", key);
 	return ret_val;
 }
 
@@ -276,9 +276,49 @@ anjuta_preferences_get_int_with_default (AnjutaPreferences *pr,
 			case GCONF_VALUE_INT:
 				ret_val = gconf_value_get_int (value);
 				break;
+			default:
+				g_warning ("Invalid gconf type for key: %s", key);
+		}
+		gconf_value_free (value);
+	}
+	return ret_val;
+}
+
+/**
+ * anjuta_preferences_get_int_with_default:
+ * @pr: A #AnjutaPreferences object
+ * @key: Property key
+ * @default_value: Default value to return if the key is not defined.
+ *
+ * Gets the value of @key as integer.
+ *
+ * Return value: Key value as integer or @default_value if the
+ * key is not defined.
+ */
+#ifdef __GNUC__
+inline
+#endif
+gint
+anjuta_preferences_get_bool_with_default (AnjutaPreferences *pr,
+                                          const gchar *key, gboolean default_value)
+{
+	gboolean ret_val;
+	GConfValue *value;
+	
+	g_return_val_if_fail (ANJUTA_IS_PREFERENCES (pr), 0);
+	g_return_val_if_fail (key != NULL, 0);
+	
+	ret_val = default_value;
+	value = gconf_client_get (pr->priv->gclient, build_key (key), NULL);
+	if (value)
+	{
+		switch (value->type)
+		{
 			case GCONF_VALUE_BOOL:
 				ret_val = gconf_value_get_bool (value);
 				break;
+			case GCONF_VALUE_INT:
+				ret_val = (gboolean) gconf_value_get_int (value);
 			default:
 				g_warning ("Invalid gconf type for key: %s", key);
 		}
@@ -354,6 +394,39 @@ anjuta_preferences_default_get_int (AnjutaPreferences *pr, const gchar *key)
 	return ret;
 }
 
+/**
+ * anjuta_preferences_default_get_int:
+ * @pr: A #AnjutaPreferences object
+ * @key: Property key
+ *
+ * Gets the default value of @key as integer. The default value of the key
+ * is the value defined in System defaults (generally installed during 
+ * program installation).
+ *
+ * Return value: Default key value as integer or 0 if the key is not defined.
+ */
+#ifdef __GNUC__
+inline
+#endif
+gboolean
+anjuta_preferences_default_get_bool (AnjutaPreferences *pr, const gchar *key)
+{
+	GConfValue *val;
+	gint ret;
+	GError *err = NULL;
+	
+	g_return_val_if_fail (ANJUTA_IS_PREFERENCES (pr), 0);
+	g_return_val_if_fail (key != NULL, 0);
+	val = gconf_client_get_default_from_schema (pr->priv->gclient, build_key (key), &err);
+	if (err) {
+		g_error_free (err);
+		return 0;
+	}
+	ret = gconf_value_get_bool (val);
+	gconf_value_free (val);
+	return ret;
+}
+
 /**
  * anjuta_preferences_set:
  * @pr: A #AnjutaPreferences object.
@@ -382,57 +455,6 @@ anjuta_preferences_set (AnjutaPreferences *pr, const gchar *key,
 	}
 }
 
-/**
- * anjuta_preferences_set_list:
- * @pr: A #AnjutaPreferences object.
- * @key: Property key.
- * @list_type: Type of each element.
- * @list: New value of the key.
- *
- * Sets a list in current session.
- */
-#ifdef __GNUC__
-inline
-#endif
-void
-anjuta_preferences_set_list (AnjutaPreferences *pr, const gchar *key,
-					         GConfValueType list_type, GSList *list)
-{
-	g_return_if_fail (ANJUTA_IS_PREFERENCES (pr));
-	g_return_if_fail (key != NULL);
-
-	gconf_client_set_list(pr->priv->gclient, build_key (key), 
-		                      list_type, list, NULL);
-}
-
-/**
- * anjuta_preferences_set_pair:
- * @pr: A #AnjutaPreferences object.
- * @key: Property key.
- * @car_type: Type of the pair's first field (car).
- * @cdr_type: Type of the pair's second field (cdr).
- * @address_of_car: Address of the car.
- * @address_of_cdr: Address of the cdr.
- *
- */
-#ifdef __GNUC__
-inline
-#endif
-gboolean
-anjuta_preferences_set_pair (AnjutaPreferences *pr, const gchar *key,
-					         GConfValueType car_type, GConfValueType cdr_type,
-                             gconstpointer address_of_car,
-                             gconstpointer address_of_cdr)
-{
-	g_return_val_if_fail (ANJUTA_IS_PREFERENCES (pr), FALSE);
-	g_return_val_if_fail (key != NULL, FALSE);
-
-	return gconf_client_set_pair (pr->priv->gclient, build_key (key),
-                                  car_type, cdr_type,
-                                  address_of_car, address_of_cdr,
-                                  NULL);
-}
-
 /**
  * anjuta_preferences_set_int:
  * @pr: A #AnjutaPreferences object.
@@ -458,10 +480,6 @@ anjuta_preferences_set_int (AnjutaPreferences *pr, const gchar *key,
 	{
 		switch (gvalue->type)
 		{
-			case GCONF_VALUE_BOOL:
-				gconf_client_set_bool (pr->priv->gclient, build_key (key),
-									   value, NULL);
-				break;
 			case GCONF_VALUE_INT:
 				gconf_client_set_int (pr->priv->gclient, build_key (key),
 									  value, NULL);
@@ -473,12 +491,56 @@ anjuta_preferences_set_int (AnjutaPreferences *pr, const gchar *key,
 	}
 	else
 	{
-		/* g_warning ("The preference key %s is not defined", key); */
+		DEBUG_PRINT ("The preference key %s is not defined", key);
 		gconf_client_set_int (pr->priv->gclient, build_key (key),
 							  value, NULL);
 	}
 }
 
+/**
+ * anjuta_preferences_set_bool:
+ * @pr: A #AnjutaPreferences object.
+ * @key: Property key.
+ * @value: Boolean value of the key.
+ *
+ * Sets the value of @key in current session.
+ */
+#ifdef __GNUC__
+inline
+#endif
+void
+anjuta_preferences_set_bool (AnjutaPreferences *pr, const gchar *key,
+							 gboolean value)
+{
+	GConfValue *gvalue;
+	
+	g_return_if_fail (ANJUTA_IS_PREFERENCES (pr));
+	g_return_if_fail (key != NULL);
+	
+	gvalue = gconf_client_get (pr->priv->gclient, build_key (key), NULL);
+	if (gvalue)
+	{
+		switch (gvalue->type)
+		{
+			case GCONF_VALUE_BOOL:
+				g_warning ("Invalid gconf type for key: %s", key);
+			case GCONF_VALUE_INT:
+				gconf_client_set_bool (pr->priv->gclient, build_key (key),
+									   value, NULL);
+				break;
+			default:
+				g_warning ("Invalid gconf type for key: %s", key);
+		}
+		gconf_value_free (gvalue);
+	}
+	else
+	{
+		DEBUG_PRINT ("The preference key %s is not defined", key);
+		gconf_client_set_bool (pr->priv->gclient, build_key (key),
+		                       value, NULL);
+	}
+}
+
 static void
 property_destroy (AnjutaProperty *property)
 {
@@ -759,12 +821,6 @@ set_property_value_as_string (AnjutaProperty *prop, const gchar *value)
 												 (prop->object), value);
 			}
 		}
-		/* FIXME: Set a standard font as default.
-		else
-		{
-			gnome_font_picker_set_font_name (GNOME_FONT_PICKER (prop->object),
-											 "A standard font");
-		}*/
 		break;
 		
 	case ANJUTA_PROPERTY_OBJECT_TYPE_FOLDER:
@@ -786,7 +842,9 @@ set_property_value_as_string (AnjutaProperty *prop, const gchar *value)
 			old_folder = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (prop->object));
 			if ((old_folder == NULL) || strcmp (old_folder, value))
 			{
-				gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (prop->object), value);
+				gchar *expand_value = anjuta_util_shell_expand (value);
+				gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (prop->object), expand_value);
+				g_free (expand_value);
 			}
 			g_free (old_folder);
 		}
@@ -794,8 +852,10 @@ set_property_value_as_string (AnjutaProperty *prop, const gchar *value)
 	case ANJUTA_PROPERTY_OBJECT_TYPE_FILE:
 		if (value)
 		{
+			gchar *expand_value = anjuta_util_shell_expand (value);
 			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (prop->object),
-															 value);
+															 expand_value);
+			g_free (expand_value);
 		}
 		break;
 	}
@@ -810,6 +870,15 @@ set_property_value_as_int (AnjutaProperty *prop, gint value)
 	g_free (text_value);	
 }
 
+static void
+set_property_value_as_bool (AnjutaProperty *prop, gboolean value)
+{
+	gchar *text_value;
+	text_value = g_strdup_printf ("%d", value);
+	set_property_value_as_string (prop, text_value);
+	g_free (text_value);	
+}
+
 static gboolean
 update_property_on_event_str (GtkWidget *widget, GdkEvent *event,
 							  gpointer user_data)
@@ -848,7 +917,7 @@ block_update_property_on_change_str (GtkWidget *widget, GdkEvent *event,
 {
 	AnjutaProperty *p = (AnjutaProperty *) user_data;
 
-	gtk_signal_handler_block_by_func (GTK_OBJECT(p->object), G_CALLBACK (update_property_on_change_str), p);
+	g_signal_handlers_block_by_func (G_OBJECT(p->object), G_CALLBACK (update_property_on_change_str), p);
 	return FALSE;
 }
 
@@ -858,7 +927,7 @@ unblock_update_property_on_change_str (GtkWidget *widget, GdkEvent *event,
 {
 	AnjutaProperty *p = (AnjutaProperty *) user_data;
 
-	gtk_signal_handler_unblock_by_func (GTK_OBJECT(p->object), G_CALLBACK (update_property_on_change_str), p);
+	g_signal_handlers_unblock_by_func (G_OBJECT(p->object), G_CALLBACK (update_property_on_change_str), p);
 	return FALSE;
 }
 
@@ -876,6 +945,20 @@ update_property_on_change_int (GtkWidget *widget, gpointer user_data)
 	anjuta_preferences_set_int (pr, p->key, val);
 }
 
+static void
+update_property_on_change_bool (GtkWidget *widget, gpointer user_data)
+{
+	AnjutaPreferences *pr;
+	AnjutaProperty *p;
+	gint val;
+	
+	pr = ANJUTA_PREFERENCES (g_object_get_data (G_OBJECT (widget),
+												"AnjutaPreferences"));
+	p = (AnjutaProperty *) user_data;
+	val = get_property_value_as_int (p);
+	anjuta_preferences_set_bool (pr, p->key, val);
+}
+
 static void
 update_property_on_change_color (GtkWidget *widget, gpointer user_data)
 {
@@ -941,7 +1024,7 @@ get_property (GConfClient *gclient, guint cnxt_id,
 		gboolean gconf_value;
 		
 		gconf_value = gconf_value_get_bool (value);
-		set_property_value_as_int (p, gconf_value);
+		set_property_value_as_bool (p, gconf_value);
 	}
 	else if (p->data_type == ANJUTA_PROPERTY_DATA_TYPE_INT)
 	{
@@ -995,7 +1078,7 @@ register_callbacks (AnjutaPreferences *pr, AnjutaProperty *p)
 			break;
 		case ANJUTA_PROPERTY_OBJECT_TYPE_TOGGLE:
 			g_signal_connect (G_OBJECT(p->object), "toggled",
-							  G_CALLBACK (update_property_on_change_int), p);
+							  G_CALLBACK (update_property_on_change_bool), p);
 			break;
 		case ANJUTA_PROPERTY_OBJECT_TYPE_COLOR:
 			g_signal_connect (G_OBJECT(p->object), "color-set",
@@ -1035,10 +1118,7 @@ preferences_foreach_callback (gchar *key, struct _AnjutaProperty *p,
 {	
 	if (p->object_type != ANJUTA_PROPERTY_OBJECT_TYPE_COMBO)
 	{
-		if (data->filter == ANJUTA_PREFERENCES_FILTER_NONE)
-			return data->callback (data->pr, key, data->callback_data);
-		else if (p->flags & data->filter)
-			return data->callback (data->pr, key, data->callback_data);
+		return data->callback (data->pr, key, data->callback_data);
 	}
 	
 	return TRUE;
@@ -1050,13 +1130,18 @@ connect_prop_to_object (AnjutaPreferences *pr, AnjutaProperty *p)
 	int gconf_value;
 	gchar *value;
 	
-	if (p->data_type == ANJUTA_PROPERTY_DATA_TYPE_BOOL ||
-		p->data_type == ANJUTA_PROPERTY_DATA_TYPE_INT)
+	if (p->data_type == ANJUTA_PROPERTY_DATA_TYPE_BOOL)
 	{	
+		gconf_value = anjuta_preferences_get_bool (pr, p->key);
+		value = g_strdup_printf ("%d", gconf_value);
+		set_property_value_as_string (p, value);
+	}
+	else if (p->data_type == ANJUTA_PROPERTY_DATA_TYPE_INT)
+	{
 		gconf_value = anjuta_preferences_get_int (pr, p->key);
 		value = g_strdup_printf ("%d", gconf_value);
 		set_property_value_as_string (p, value);
-	} 
+    }
 	else 
 	{
 		value = anjuta_preferences_get (pr, p->key);
@@ -1250,9 +1335,7 @@ anjuta_preferences_register_property_custom (AnjutaPreferences *pr,
 			   data_type == ANJUTA_PROPERTY_DATA_TYPE_BOOL) ||
 			  (value->type == GCONF_VALUE_INT &&
 			   data_type == ANJUTA_PROPERTY_DATA_TYPE_INT) ||
-			  (value->type == GCONF_VALUE_STRING &&
-			   data_type != ANJUTA_PROPERTY_DATA_TYPE_BOOL &&
-			   data_type != ANJUTA_PROPERTY_DATA_TYPE_INT)))
+			  (value->type == GCONF_VALUE_STRING)))
 		{
 			gconf_client_unset (pr->priv->gclient, build_key (key), NULL);
 		}
@@ -1375,35 +1458,42 @@ anjuta_preferences_register_property_from_string (AnjutaPreferences *pr,
 }
 
 /**
- * anjuta_preferences_register_all_properties_from_glade_xml:
+ * anjuta_preferences_register_all_properties_from_builder_xml:
  * @pr: a #AnjutaPreferences Object
- * @gxml: GladeXML object containing the properties widgets.
- * @parent: Parent widget in the gxml object
+ * @builder: GtkBuilder object containing the properties widgets.
+ * @parent: Parent widget in the builder object
  *
  * This will register all the properties names of the format described above
  * without considering the UI. Useful if you have the widgets shown elsewhere
  * but you want them to be part of preferences system.
  */
 void
-anjuta_preferences_register_all_properties_from_glade_xml (AnjutaPreferences *pr,
-														   GladeXML *gxml,
-														   GtkWidget *parent)
+anjuta_preferences_register_all_properties_from_builder_xml (AnjutaPreferences *pr,
+                                                             GtkBuilder *builder,
+                                                             GtkWidget *parent)
 {
-	GList *widgets;
-	GList *node;
+	GSList *widgets;
+	GSList *node;
 	
 	g_return_if_fail (ANJUTA_IS_PREFERENCES (pr));
-	g_return_if_fail (gxml != NULL);
+	g_return_if_fail (builder != NULL);
 	
-	widgets = glade_xml_get_widget_prefix (gxml, "preferences_");
-	node = widgets;
-	while (node)
+	widgets = gtk_builder_get_objects (builder);
+	for (node = widgets; node != NULL; node = g_slist_next (node))
 	{
 		const gchar *name;
+		const gchar *property;
 		GtkWidget *widget, *p;
 		gboolean cont_flag = FALSE;
+
+		if (!GTK_IS_WIDGET (node->data))
+			continue;
 		
 		widget = node->data;
+		name = gtk_widget_get_name (widget);
+
+		if (!g_str_has_prefix (name, PREFERENCE_PROPERTY_PREFIX))
+			continue;
 		
 		p = gtk_widget_get_parent (widget);
 		/* Added only if it's a desendend child of the parent */
@@ -1416,21 +1506,12 @@ anjuta_preferences_register_all_properties_from_glade_xml (AnjutaPreferences *pr
 			}
 			p = gtk_widget_get_parent (p);
 		}
-		if (cont_flag == TRUE)
-		{
-			node = g_list_next (node);
+		if (cont_flag)
 			continue;
-		}
-		
-		name = glade_get_widget_name (widget);
-		if (strncmp (name, PREFERENCE_PROPERTY_PREFIX,
-                     strlen (PREFERENCE_PROPERTY_PREFIX)) == 0)
-		{
-			const gchar *property = &name[strlen (PREFERENCE_PROPERTY_PREFIX)];
-			anjuta_preferences_register_property_from_string (pr, widget,
-															  property);
-		}
-		node = g_list_next (node);
+
+		property = &name[strlen (PREFERENCE_PROPERTY_PREFIX)];
+		anjuta_preferences_register_property_from_string (pr, widget,
+		                                                  property);
 	}
 }
 
@@ -1477,14 +1558,12 @@ anjuta_preferences_reset_defaults (AnjutaPreferences * pr)
  */
 void
 anjuta_preferences_foreach (AnjutaPreferences *pr,
-							AnjutaPreferencesFilterType filter,
 							AnjutaPreferencesCallback callback,
 							gpointer data)
 {
 	struct _AnjutaPreferencesForeachData foreach_data;
 	
 	foreach_data.pr = pr;
-	foreach_data.filter = filter;
 	foreach_data.callback = callback;
 	foreach_data.callback_data = data;
 	
@@ -1497,16 +1576,16 @@ anjuta_preferences_foreach (AnjutaPreferences *pr,
 /**
  * anjuta_preferences_add_page:
  * @pr: a #AnjutaPreferences object
- * @gxml: #GladeXML object containing the preferences page
- * @glade_widget_name: Page widget name (as give with glade interface editor).
+ * @builder: #GtkBuilder object containing the preferences page
+ * @gwidget_name: Page widget name (as give with glade interface editor).
  * The widget will be searched with the given name and detached
  * (that is, removed from the container, if present) from it's parent.
  * @icon_filename: File name (of the form filename.png) of the icon representing
  * the preference page.
  * 
  * Add a page to the preferences sytem.
- * gxml is the GladeXML object of the glade dialog containing the page widget.
- * The glade dialog will contain the layout of the preferences widgets.
+ * builder is the GtkBuilder object of the dialog containing the page widget.
+ * The dialog will contain the layout of the preferences widgets.
  * The widgets which are preference widgets (e.g. toggle button) should have
  * widget names of the form:
  *
@@ -1531,10 +1610,11 @@ anjuta_preferences_foreach (AnjutaPreferences *pr,
  * anjuta preferences dialogs and study the widget names.
  */
 void
-anjuta_preferences_add_page (AnjutaPreferences* pr, GladeXML *gxml,
-							 const gchar* glade_widget_name,
-							 const gchar* title,
-							 const gchar *icon_filename)
+anjuta_preferences_add_from_builder (AnjutaPreferences* pr, 
+                                     GtkBuilder* builder,
+                                     const gchar* widget_name,
+                                     const gchar* title,
+                                     const gchar *icon_filename)
 {
 	GtkWidget *parent;
 	GtkWidget *page;
@@ -1542,10 +1622,10 @@ anjuta_preferences_add_page (AnjutaPreferences* pr, GladeXML *gxml,
 	gchar *image_path;
 	
 	g_return_if_fail (ANJUTA_IS_PREFERENCES (pr));
-	g_return_if_fail (glade_widget_name != NULL);
+	g_return_if_fail (widget_name != NULL);
 	g_return_if_fail (icon_filename != NULL);
 	
-	page = glade_xml_get_widget (gxml, glade_widget_name);
+	page = GTK_WIDGET(gtk_builder_get_object (builder, widget_name));
 	g_object_ref (page);
 	g_return_if_fail (GTK_IS_WIDGET (page));
 	parent = gtk_widget_get_parent (page);
@@ -1566,8 +1646,8 @@ anjuta_preferences_add_page (AnjutaPreferences* pr, GladeXML *gxml,
 	image_path = anjuta_res_get_pixmap_file (icon_filename);
 	pixbuf = gdk_pixbuf_new_from_file (image_path, NULL);
 	anjuta_preferences_dialog_add_page (ANJUTA_PREFERENCES_DIALOG (pr->priv->prefs_dialog),
-										glade_widget_name, title, pixbuf, page);
-	anjuta_preferences_register_all_properties_from_glade_xml (pr, gxml, page);
+										widget_name, title, pixbuf, page);
+	anjuta_preferences_register_all_properties_from_builder_xml (pr, builder, page);
 	g_object_unref (page);
 	g_free (image_path);
 	g_object_unref (pixbuf);
@@ -1704,6 +1784,10 @@ anjuta_preferences_instance_init (AnjutaPreferences *pr)
 	pr->priv->properties = g_hash_table_new_full (g_str_hash, g_str_equal,
 												  g_free, 
 												  (GDestroyNotify) property_destroy);
+	pr->priv->notifications = g_hash_table_new_full (g_int_hash,
+	                                                 g_int_equal,
+	                                                 g_free, 
+	                                                 NULL);
 	
 	pr->priv->gclient = gconf_client_get_default();
 	gconf_client_add_dir (pr->priv->gclient, GCONF_KEY_PREFIX,
@@ -1743,16 +1827,93 @@ AnjutaPreferences *
 anjuta_preferences_new (AnjutaPluginManager *plugin_manager)
 {
 	AnjutaPreferences *pr;
-	
-	pr = g_object_new (ANJUTA_TYPE_PREFERENCES, NULL);
-	pr->priv->plugin_manager = g_object_ref (plugin_manager);
-	
-	return pr;
+
+	if (!default_preferences)
+	{
+		pr = g_object_new (ANJUTA_TYPE_PREFERENCES, NULL);
+		pr->priv->plugin_manager = g_object_ref (plugin_manager);
+		default_preferences = pr;
+		return pr;
+	}
+	else
+		return default_preferences;
 	
 }
 
 /**
- * anjuta_preferences_notify_add:
+ * anjuta_preferences_default:
+ * 
+ * Get the default instace of anjuta preferences
+ * 
+ * Return value: A #AnjutaPreferences object.
+ */
+AnjutaPreferences *anjuta_preferences_default ()
+{
+	return default_preferences;
+}
+
+static void
+gconf_notify (GConfEngine* conf,
+              guint id,
+              GConfEntry* entry,
+              gpointer user_data)
+{
+	AnjutaPreferences* prefs = anjuta_preferences_default();
+	gpointer notify =
+		g_hash_table_lookup (prefs->priv->notifications,
+		                     &id);
+	
+	if (entry->value)
+	{
+		const gchar* real_key = unbuild_key (entry->key);
+		switch (entry->value->type)
+		{
+			case GCONF_VALUE_INT:
+			{
+				AnjutaPreferencesNotifyInt int_func = notify;
+				int_func (prefs, real_key,
+				          gconf_value_get_int (entry->value), user_data);
+				break;
+			}
+			case GCONF_VALUE_STRING:
+			{
+				AnjutaPreferencesNotify str_func = notify;
+				str_func (prefs, real_key, 
+				          gconf_value_get_string (entry->value), user_data);
+				break;
+			}
+			case GCONF_VALUE_BOOL:
+			{
+				AnjutaPreferencesNotifyBool bool_func = notify;
+				bool_func (prefs, real_key,
+				           gconf_value_get_bool (entry->value), user_data);
+				break;
+			}
+			default:
+				g_warning ("Notification for unknown type of key: %s", entry->key);
+		}
+	}
+}
+
+guint 
+anjuta_preferences_notify_add_int (AnjutaPreferences *pr,
+                                   const gchar *key,
+                                   AnjutaPreferencesNotifyInt func,
+                                   gpointer data,
+                                   GFreeFunc destroy_notify)
+{
+	guint* id = g_new0 (guint, 1);
+	*id = gconf_client_notify_add (pr->priv->gclient,
+	                              build_key (key),
+	                              (GConfClientNotifyFunc) gconf_notify, data, destroy_notify, NULL);
+	g_hash_table_insert (pr->priv->notifications,
+	                     id,
+	                     func);
+	return *id;
+}
+
+/**
+ * anjuta_preferences_notify_add_string:
  * @pr: A #AnjutaPreferences object.
  * @key: Key to monitor.
  * @func: User callback function.
@@ -1765,18 +1926,41 @@ anjuta_preferences_new (AnjutaPluginManager *plugin_manager)
  *
  * Return value: Notify ID.
  */
-guint
-anjuta_preferences_notify_add (AnjutaPreferences *pr,
-							   const gchar *key,
-							   GConfClientNotifyFunc func,
-							   gpointer data,
-							   GFreeFunc destroy_notify)
+guint 
+anjuta_preferences_notify_add_string (AnjutaPreferences *pr,
+                                      const gchar *key,
+                                      AnjutaPreferencesNotify func,
+                                      gpointer data,
+                                      GFreeFunc destroy_notify)
 {
-	return gconf_client_notify_add (pr->priv->gclient,
-									build_key (key),
-									func, data, destroy_notify, NULL);
+	guint* id = g_new0 (guint, 1);
+	*id = gconf_client_notify_add (pr->priv->gclient,
+	                              build_key (key),
+	                              (GConfClientNotifyFunc)gconf_notify, data, destroy_notify, NULL);
+	g_hash_table_insert (pr->priv->notifications,
+	                     id,
+	                     func);
+	return *id;
 }
 
+guint 
+anjuta_preferences_notify_add_bool (AnjutaPreferences *pr,
+                                    const gchar *key,
+                                    AnjutaPreferencesNotifyBool func,
+                                    gpointer data,
+                                    GFreeFunc destroy_notify)
+{
+	guint* id = g_new0 (guint, 1);
+	*id = gconf_client_notify_add (pr->priv->gclient,
+	                              build_key (key),
+	                              (GConfClientNotifyFunc)gconf_notify, data, destroy_notify, NULL);
+	g_hash_table_insert (pr->priv->notifications,
+	                     id,
+	                     func);
+	return *id;
+}
+
+
 /**
  * anjuta_preferences_notify_remove:
  * @pr: A #AnjutaPreferences object.
@@ -1803,63 +1987,3 @@ anjuta_preferences_get_prefix (AnjutaPreferences *pr)
 {
 	return PREFERENCE_PROPERTY_PREFIX;
 }
-
-/**
- * anjuta_preferences_dir_exists:
- * @pr: A #AnjutaPreferences object.
- * @dir: Directory to checkfor.
- *
- * Return value: TRUE if dir exists.
- */
-#ifdef __GNUC__
-inline
-#endif
-gboolean
-anjuta_preferences_dir_exists (AnjutaPreferences *pr, const gchar *dir)
-{
-	g_return_val_if_fail (ANJUTA_IS_PREFERENCES (pr), FALSE);
-	g_return_val_if_fail (dir != NULL, FALSE);
-
-	return gconf_client_dir_exists(pr->priv->gclient, build_key (dir), NULL);
-}
-
-/**
- * anjuta_preferences_add_dir:
- * @pr: A #AnjutaPreferences object.
- * @dir: Directory to add to the list.
- * @preload: Degree of preload.
- *
- * Add a directory to the list of directories the GConfClient.
- */
-#ifdef __GNUC__
-inline
-#endif
-void
-anjuta_preferences_add_dir (AnjutaPreferences *pr, const gchar *dir, 
-                               GConfClientPreloadType preload)
-{
-	g_return_if_fail (ANJUTA_IS_PREFERENCES (pr));
-	g_return_if_fail (dir != NULL);
-
-	gconf_client_add_dir(pr->priv->gclient, build_key (dir), 
-	                     preload, NULL);
-}
-
-/**
- * anjuta_preferences_remove_dir:
- * @pr: A #AnjutaPreferences object.
- * @dir: Directory to remove from the list.
- *
- * Remove a directory from the list of directories.
- */
-#ifdef __GNUC__
-inline
-#endif
-void
-anjuta_preferences_remove_dir (AnjutaPreferences *pr, const gchar *dir)
-{
-	g_return_if_fail (ANJUTA_IS_PREFERENCES (pr));
-	g_return_if_fail (dir != NULL);
-
-	gconf_client_remove_dir(pr->priv->gclient, build_key (dir), NULL);
-}
