@@ -52,6 +52,8 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 
+#include <gconf/gconf-client.h>
+
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/anjuta-debug.h>
 
@@ -82,20 +84,20 @@ anjuta_util_from_file_to_file (GInputStream *istream,
 	
 	if (error)
 	{
-		g_warning (error->message);
+		g_warning ("%s", error->message);
 		g_error_free (error);
 		error = NULL;
 	}
 	
 	if (!g_output_stream_close (ostream, NULL, &error))
 	{
-		g_warning (error->message);
+		g_warning ("%s", error->message);
 		g_error_free (error);
 		error = NULL;
 	}
 	if (!g_input_stream_close (istream, NULL, &error))
 	{
-		g_warning (error->message);
+		g_warning ("%s", error->message);
 		g_error_free (error);
 	}
 }
@@ -168,16 +170,6 @@ anjuta_util_string_from_color (guint16 r, guint16 g, guint16 b)
 	return g_strdup_printf("#%02x%02x%02x", r >> 8, g >> 8, b >> 8);
 }
 
-/* Get a GdkColor from preferences. Free the color with gdk_color_free() */
-GdkColor*
-anjuta_util_convert_color(AnjutaPreferences* prefs, const gchar* pref_name)
-{
-	GdkColor* color = g_new0(GdkColor, 1);
-	gchar* color_string = anjuta_preferences_get(prefs, pref_name);
-	gdk_color_parse(color_string, color);
-	return color;
-}
-
 GtkWidget* 
 anjuta_util_button_new_with_stock_image (const gchar* text,
 										 const gchar* stock_id)
@@ -859,14 +851,14 @@ anjuta_util_create_dir (const gchar* path)
  * Returns: A newly allocated string that is the path to the shell.
  */
 /* copied from deprecated gnome_util_user_shell in libgnome */
-char *
+gchar *
 anjuta_util_user_shell (void)
 {
 #ifndef G_OS_WIN32
 	struct passwd *pw;
-	int i;
-	const char *shell;
-	const char shells [][14] = {
+	gint i;
+	const gchar *shell;
+	const gchar shells [][14] = {
 		/* Note that on some systems shells can also
 		 * be installed in /usr/bin */
 		"/bin/bash", "/usr/bin/bash",
@@ -874,7 +866,7 @@ anjuta_util_user_shell (void)
 		"/bin/tcsh", "/usr/bin/tcsh",
 		"/bin/ksh", "/usr/bin/ksh",
 		"/bin/csh", "/bin/sh"
-											};
+	};
 
 	if (geteuid () == getuid () &&
 	    getegid () == getgid ()) {
@@ -908,7 +900,7 @@ anjuta_util_user_shell (void)
  	 * and System32 directories, so it should always find either cmd.exe
  	 * or command.com.
  	 */
-	char *retval = g_find_program_in_path ("cmd.exe");
+	gchar *retval = g_find_program_in_path ("cmd.exe");
 
 	if (retval == NULL)
 		retval = g_find_program_in_path ("command.com");
@@ -919,6 +911,80 @@ anjuta_util_user_shell (void)
 #endif
 }
 
+/**
+ * anjuta_util_user_terminal:
+ *
+ * Retrieves the user's preferred terminal.
+ *
+ * Returns: A newly allocated strings list. The first argument is the terminal 
+ * program name. The following are the arguments needed to execute
+ * a command. The list has to be freed with g_strfreev
+ */
+/* copied from deprecated gnome_execute_terminal in libgnome */
+gchar **
+anjuta_util_user_terminal (void)
+{
+#ifndef G_OS_WIN32
+	GConfClient *client;
+	gchar *terminal = NULL;
+	gchar **argv = NULL;
+	static const gchar *terms[] = {
+		"xdg-terminal",
+		"gnome-terminal",
+		"nxterm",
+		"color-xterm",
+		"rxvt",
+		"xterm",
+		"dtterm",
+		NULL
+	};
+	const gchar **term;
+
+	client = gconf_client_get_default ();
+	terminal = gconf_client_get_string (client, "/desktop/gnome/applications/terminal/exec", NULL);
+	g_object_unref (client);
+	
+	if (terminal)
+	{
+		gchar *command_line;
+		gchar *exec_flag;
+
+		exec_flag = gconf_client_get_string (client, "/desktop/gnome/applications/terminal/exec_arg", NULL);
+		command_line = g_strconcat (terminal, " ", exec_flag, NULL);
+
+		g_shell_parse_argv (command_line, NULL, &argv, NULL);
+		g_free (terminal);
+		g_free (exec_flag);
+
+		return argv;
+	}
+
+
+	/* Search for common ones */
+	for (term = terms; *term != NULL; term++)
+	{
+		terminal = g_find_program_in_path (*term);
+		if (terminal != NULL) break;
+	}
+
+	/* Try xterm */
+	g_warning (_("Cannot find a terminal, using "
+			"xterm, even if it may not work"));
+	terminal = g_strdup ("xterm");
+
+	argv = g_new0 (char *, 3);
+	argv[0] = terminal;
+	/* Note that gnome-terminal takes -x and
+	 * as -e in gnome-terminal is broken we use that. */
+	argv[1] = g_strdup (term == &terms[2] ? "-x" : "-e");
+
+	return argv;
+#else
+	g_warning ("anjuta_util_user_terminal: Not implemented");
+	return NULL;
+#endif
+}
+
 pid_t
 anjuta_util_execute_shell (const gchar *dir, const gchar *command)
 {
@@ -944,6 +1010,40 @@ anjuta_util_execute_shell (const gchar *dir, const gchar *command)
 	if (pid < 0)
 		g_warning (_("Cannot execute command: %s (using shell %s)\n"), command, shell);
 	g_free (shell);
+
+	// Anjuta will take care of child exit automatically.
+	return pid;
+}
+
+pid_t
+anjuta_util_execute_terminal_shell (const gchar *dir, const gchar *command)
+{
+	pid_t pid;
+	gchar *shell;
+	gchar **term_argv;
+	gint err;
+	
+	g_return_val_if_fail (command != NULL, -1);
+	
+	shell = anjuta_util_user_shell ();
+	term_argv = anjuta_util_user_terminal ();
+	pid = fork();
+	if (pid == 0)
+	{
+		if(dir)
+		{
+			anjuta_util_create_dir (dir);
+			err = chdir (dir);
+		}
+		execlp (term_argv[0], term_argv[0], term_argv[1], shell, "-c", command, NULL);
+		g_warning (_("Cannot execute command: %s (using shell %s)\n"), command, shell);
+		_exit(1);
+	}
+	if (pid < 0)
+		g_warning (_("Cannot execute command: %s (using shell %s)\n"), command, shell);
+	g_free (shell);
+	g_strfreev (term_argv);
+
 	// Anjuta will take care of child exit automatically.
 	return pid;
 }
@@ -1944,9 +2044,8 @@ anjuta_util_convert_gfile_list_to_path_list (GList *list)
 	gchar *path;
 	
 	path_list = NULL;
-	current_file = list;
 	
-	while (current_file)
+	for (current_file = list; current_file != NULL; current_file = g_list_next (current_file))
 	{
 		path = g_file_get_path (current_file->data);
 		
@@ -1958,3 +2057,125 @@ anjuta_util_convert_gfile_list_to_path_list (GList *list)
 	return path_list;
 }
 
+GList *
+anjuta_util_convert_gfile_list_to_relative_path_list (GList *list, 
+                                                      const gchar *parent)
+{
+	GFile *parent_file;
+	GList *path_list;
+	GList *current_file;
+	gchar *path;
+	
+	parent_file = g_file_new_for_path (parent);
+	path_list = NULL;
+	
+	if (parent_file)
+	{		
+		for (current_file = list; current_file != NULL; current_file = g_list_next (current_file))
+		{
+			path = g_file_get_relative_path (parent_file, current_file->data);
+			
+			/* Ignore files with invalid paths */
+			if (path)
+				path_list = g_list_append (path_list, path);
+		}
+		
+		g_object_unref (parent_file);
+	}
+	
+	return path_list;
+}
+
+
+/**
+ * anjuta_util_builder_new:
+ * @filename: Builder file name to open
+ * @error: Optional error object, if NULL display a dialog if the file is missing
+ *
+ * Create a new GtkBuilder object and load the file in it. Display an error
+ * if the file is missing. Use a dialog if error is NULL, just a warning
+ * if the error can be reported.
+  *
+ * Returns: The new GtkBuilder object
+ */
+GtkBuilder *
+anjuta_util_builder_new (const gchar *filename, GError **error)
+{
+	GtkBuilder *bxml = gtk_builder_new ();
+	GError *err = NULL;
+
+	/* Load glade file */
+	if (!gtk_builder_add_from_file (bxml, filename, &err))
+	{
+		g_object_unref (bxml);
+		bxml = NULL;
+
+		/* Display the error to the user if it cannot be reported to the caller */
+		if (error == NULL)
+		{
+			anjuta_util_dialog_error (NULL, _("Unable to load user interface file: %s"), err->message);
+		}
+		else
+		{
+			g_warning ("Couldn't load builder file: %s", err->message);
+		}
+		g_propagate_error (error, err);
+	}
+
+	/* Tag the builder object with the filename to allow better error message 
+	 * with the following function */
+	if (bxml != NULL)
+	{
+		g_object_set_data_full (G_OBJECT (bxml), "filename", g_strdup (filename), g_free);
+	}
+
+	return bxml;
+}
+
+/**
+ * anjuta_util_builder_get_objects:
+ * @builder: Builder object
+ * @first_widget: Name of first widget to get
+ * ...: Address to store the first widget pointer, followed optionally by
+ *		more name/pointer pairs, followed by NULL
+ *
+ * Create a new GtkBuilder object and load the file in it. Display an error
+ * if the file is missing. Use a dialog if error is NULL, just a warning
+ * if the error can be reported.
+  *
+ * Returns: TRUE is everything works as expected.
+ */
+gboolean
+anjuta_util_builder_get_objects (GtkBuilder *builder, const gchar *first_widget,...)
+{
+	va_list args;
+	const gchar *name;
+	GObject **object_ptr;
+	gboolean missing = FALSE;
+
+	va_start (args, first_widget);
+
+	for (name = first_widget; name; name = va_arg (args, char *))
+	{
+		object_ptr = va_arg (args, void *);
+		*object_ptr = gtk_builder_get_object (builder, name);
+
+		/* Object not found, display a warning */
+		if (!*object_ptr)
+		{
+			const gchar *filename = (const gchar *)g_object_get_data (G_OBJECT (builder), "filename");
+			if (filename)
+			{
+				g_warning ("Missing widget '%s' in file %s", name, filename);
+			}
+			else
+			{
+				g_warning("Missing widget '%s'", name);
+			}
+			missing = TRUE;
+        }
+	}
+	 va_end (args);
+
+	return !missing;
+}
