@@ -21,8 +21,8 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include <gnome.h>
-#include <glade/glade.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/anjuta-plugin.h>
@@ -41,9 +41,9 @@
 #include <libanjuta/interfaces/ianjuta-project-manager.h>
 #include <glib/gi18n.h>
 
-#define GLADE_FILE_SEARCH_REPLACE PACKAGE_DATA_DIR"/glade/anjuta-search.glade"
+#define BUILDER_FILE_SEARCH_REPLACE PACKAGE_DATA_DIR"/glade/anjuta-search.ui"
 
-/* LibGlade's auto-signal-connect will connect to these signals.
+/* GtkBuilder's auto-signal-connect will connect to these signals.
  * Do not declare them static.
  */
 gboolean
@@ -118,7 +118,7 @@ AnjutaUtilStringMap search_action_strings[] = {
 
 typedef struct _SearchReplaceGUI
 {
-	GladeXML *xml;
+	GtkBuilder *bxml;
 	GtkWidget *dialog;
 	gboolean showing;
 } SearchReplaceGUI;
@@ -654,15 +654,13 @@ write_message_pane(IAnjutaMessageView* view, FileBuffer *fb, SearchEntry *se,
 	int line = mi->line;
 	gchar *buf;
 	gchar *path = NULL;
-	
-	if (se->type == SE_FILE)
-		++line;
+
 	if (fb->file != NULL)
 	{
 		gchar *dir_uri = NULL;	
 		
 		anjuta_shell_get (ANJUTA_PLUGIN(sr->docman)->shell,
-							  "project_root_uri", G_TYPE_STRING,
+							  IANJUTA_PROJECT_MANAGER_PROJECT_ROOT_URI, G_TYPE_STRING,
 							  &dir_uri, NULL);
 		if (dir_uri != NULL)
 		{
@@ -724,7 +722,7 @@ on_message_clicked (GObject* object, gchar* message, gpointer data)
 	line = atoi(nline);
 	
 	anjuta_shell_get (ANJUTA_PLUGIN(sr->docman)->shell,
-	                  "project_root_uri", G_TYPE_STRING,
+	                  IANJUTA_PROJECT_MANAGER_PROJECT_ROOT_URI, G_TYPE_STRING,
 	                  &project_uri, NULL);
 	
 	if (project_uri && strlen(project_uri))
@@ -1278,6 +1276,7 @@ void translate_dialog_strings (AnjutaUtilStringMap labels[])
 static gboolean
 create_dialog(void)
 {
+	GError* error = NULL;
 	GladeWidget *w;
 	GtkWidget *widget;
 	int i;
@@ -1286,15 +1285,17 @@ create_dialog(void)
 	if (NULL != sg) return TRUE;
 	sg = g_new0(SearchReplaceGUI, 1);
 
-	if (NULL == (sg->xml = glade_xml_new(GLADE_FILE_SEARCH_REPLACE,
-		SEARCH_REPLACE_DIALOG, NULL)))
+	sg->bxml = gtk_builder_new ();
+	if (!gtk_builder_add_from_file (sg->bxml, BUILDER_FILE_SEARCH_REPLACE, &error))
 	{
 		anjuta_util_dialog_error(NULL, _("Unable to build user interface for Search And Replace"));
 		g_free(sg);
 		sg = NULL;
+		g_error_free (error);
 		return FALSE;
 	}
-	sg->dialog = glade_xml_get_widget(sg->xml, SEARCH_REPLACE_DIALOG);
+
+	sg->dialog = GTK_WIDGET (gtk_builder_get_object (sg->bxml, SEARCH_REPLACE_DIALOG));
 	/* gtk_window_set_transient_for (GTK_WINDOW(sg->dialog)
 	  , GTK_WINDOW(app->widgets.window)); */
 		
@@ -1309,7 +1310,7 @@ create_dialog(void)
 	for (i=0; NULL != glade_widgets[i].name; ++i)
 	{
 		w = &(glade_widgets[i]);
-		w->widget = glade_xml_get_widget(sg->xml, w->name);
+		w->widget = GTK_WIDGET (gtk_builder_get_object (sg->bxml, w->name));
 		if (GE_COMBO_ENTRY == w->type)
 		{
 			/* Get child of GtkComboBoxEntry */
@@ -1342,8 +1343,8 @@ create_dialog(void)
 
 	search_preferences_initialize_setting_treeview(sg->dialog);
 	search_preferences_init();
-	
-	glade_xml_signal_autoconnect(sg->xml);
+
+	gtk_builder_connect_signals (sg->bxml, NULL);
 	return TRUE;
 }
 
@@ -1869,7 +1870,7 @@ void search_replace_find_usage(const gchar *symbol)
 	g_object_get(G_OBJECT(sr->docman), "shell", &shell, NULL);
 	
 	anjuta_shell_get (shell,
-					  "project_root_uri", G_TYPE_STRING,
+					  IANJUTA_PROJECT_MANAGER_PROJECT_ROOT_URI, G_TYPE_STRING,
 					  &project_root_uri, NULL);
 	
 	sr->search.range.type =
@@ -1999,6 +2000,7 @@ anjuta_search_replace_activate (gboolean replace, gboolean project)
 {
 	GtkWidget *notebook;
 	GtkWidget *search_entry;
+	GtkWidget *search_button;
 	IAnjutaDocument *doc;
 	IAnjutaEditor *te;
 
@@ -2011,6 +2013,7 @@ anjuta_search_replace_activate (gboolean replace, gboolean project)
 	reset_flags_and_search_button();
 	
 	search_entry = sr_get_gladewidget(SEARCH_STRING)->widget;
+	search_button = sr_get_gladewidget(SEARCH_BUTTON)->widget;
 	doc = ianjuta_document_manager_get_current_document(sr->docman, NULL);
 	te = (IANJUTA_IS_EDITOR (doc)) ? IANJUTA_EDITOR (doc) : NULL;
 	if (te && search_entry && sr->search.range.type != SR_SELECTION)
@@ -2071,5 +2074,12 @@ anjuta_search_replace_activate (gboolean replace, gboolean project)
 	/* Show the dialog */
 	if (search_entry)
 	gtk_widget_grab_focus (search_entry);
+
+	/* Set default button */
+	if (search_button)
+	{
+		GTK_WIDGET_SET_FLAGS (search_button, GTK_CAN_DEFAULT);
+		gtk_widget_grab_default(search_button);
+	}
 	show_dialog();
 }
