@@ -351,33 +351,6 @@ enable_view_signals (SymbolDBPlugin *sdb_plugin, gboolean enable, gboolean force
 	}
 }
 
-static void
-on_editor_buffer_symbol_update_scan_end (SymbolDBEngine *dbe, gint process_id, 
-										  gpointer data)
-{
-	SymbolDBPlugin *sdb_plugin;
-	gint i;
-	
-	sdb_plugin = ANJUTA_PLUGIN_SYMBOL_DB (data);	
-	
-	/* search for the proc id */
-	for (i = 0; i < sdb_plugin->buffer_update_ids->len; i++)
-	{
-		if (g_ptr_array_index (sdb_plugin->buffer_update_ids, i) == GINT_TO_POINTER (process_id))
-		{
-			gchar *str;
-			/* hey we found it */
-			/* remove both the items */
-			g_ptr_array_remove_index (sdb_plugin->buffer_update_ids, i);
-			
-			str = (gchar*)g_ptr_array_remove_index (sdb_plugin->buffer_update_files, 
-													i);
-			/* we can now free it */
-			g_free (str);			
-		}
-	}
-}
-
 static gboolean
 on_editor_buffer_symbols_update_timeout (gpointer user_data)
 {
@@ -490,6 +463,76 @@ on_editor_buffer_symbols_update_timeout (gpointer user_data)
 	return proc_id > 0 ? TRUE : FALSE;
 }
 
+static void
+on_editor_buffer_symbol_update_scan_end (SymbolDBEngine *dbe, gint process_id, 
+										  gpointer data)
+{
+	SymbolDBPlugin *sdb_plugin;
+	gint i;
+	
+	sdb_plugin = ANJUTA_PLUGIN_SYMBOL_DB (data);	
+	
+	/* search for the proc id */
+	for (i = 0; i < sdb_plugin->buffer_update_ids->len; i++)
+	{
+		if (g_ptr_array_index (sdb_plugin->buffer_update_ids, i) == GINT_TO_POINTER (process_id))
+		{
+			gchar *str;
+			/* hey we found it */
+			/* remove both the items */
+			g_ptr_array_remove_index (sdb_plugin->buffer_update_ids, i);
+			
+			str = (gchar*)g_ptr_array_remove_index (sdb_plugin->buffer_update_files, 
+													i);
+			/* we can now free it */
+			g_free (str);			
+		}
+	}
+
+	/* was the updating of view-locals symbols blocked while we were scanning?
+	 * e.g. was the editor switched? */
+	if (sdb_plugin->buffer_update_semaphore == TRUE)
+	{
+		GFile *file;
+		gchar *local_path;
+		gboolean tags_update;
+		if (!IANJUTA_IS_EDITOR (sdb_plugin->current_editor))
+			return;
+	
+		file = ianjuta_file_get_file (IANJUTA_FILE (sdb_plugin->current_editor), 
+		    NULL);
+	
+		if (file == NULL)
+			return;
+
+		local_path = g_file_get_path (file);
+	
+		if (local_path == NULL)
+		{
+			g_critical ("local_path == NULL");
+			return;
+		}	
+
+		symbol_db_view_locals_update_list (
+					SYMBOL_DB_VIEW_LOCALS (sdb_plugin->dbv_view_tree_locals),
+					 sdb_plugin->sdbe_project, local_path, FALSE);
+
+		/* add a default timeout to the updating of buffer symbols */	
+		tags_update = anjuta_preferences_get_bool (sdb_plugin->prefs, BUFFER_AUTOSCAN);
+		
+		if (tags_update)
+		{
+			sdb_plugin->buf_update_timeout_id = 
+					g_timeout_add_seconds (TIMEOUT_INTERVAL_SYMBOLS_UPDATE,
+										   on_editor_buffer_symbols_update_timeout,
+										   sdb_plugin);
+		}		
+		
+		g_free (local_path);
+		sdb_plugin->buffer_update_semaphore = FALSE;
+	}	 
+}
+
 static void
 on_editor_destroy (SymbolDBPlugin *sdb_plugin, IAnjutaEditor *editor)
 {
@@ -639,10 +682,33 @@ value_added_current_editor (AnjutaPlugin *plugin, const char *name,
 		g_critical ("local_path == NULL");
 		return;
 	}	
+
+	/* we can have a weird behaviour here if we're not paying the right attention:
+	 * A timeout scan could have been launched and a millisecond later the user could
+	 * have switched editor: we'll be getting the symbol inserted in the previous
+	 * editor into the new one's view.
+	 */
+	if (sdb_plugin->buffer_update_files->len > 0)
+	{
+		sdb_plugin->buffer_update_semaphore = TRUE;
+	}
+	else
+	{
+		symbol_db_view_locals_update_list (
+					SYMBOL_DB_VIEW_LOCALS (sdb_plugin->dbv_view_tree_locals),
+					 sdb_plugin->sdbe_project, local_path, FALSE);
+
+		/* add a default timeout to the updating of buffer symbols */	
+		tags_update = anjuta_preferences_get_bool (sdb_plugin->prefs, BUFFER_AUTOSCAN);
 				
-	symbol_db_view_locals_update_list (
-				SYMBOL_DB_VIEW_LOCALS (sdb_plugin->dbv_view_tree_locals),
-				 sdb_plugin->sdbe_project, local_path, FALSE);
+		if (tags_update)
+		{
+			sdb_plugin->buf_update_timeout_id = 
+					g_timeout_add_seconds (TIMEOUT_INTERVAL_SYMBOLS_UPDATE,
+										   on_editor_buffer_symbols_update_timeout,
+										   plugin);
+		}		
+	}
 				 
 	if (g_hash_table_lookup (sdb_plugin->editor_connected, editor) == NULL)
 	{
@@ -673,14 +739,6 @@ value_added_current_editor (AnjutaPlugin *plugin, const char *name,
 	g_free (uri);
 	g_free (local_path);
 	
-	/* add a default timeout to the updating of buffer symbols */	
-	tags_update = anjuta_preferences_get_bool (sdb_plugin->prefs, BUFFER_AUTOSCAN);
-				
-	if (tags_update)
-		sdb_plugin->buf_update_timeout_id = 
-				g_timeout_add_seconds (TIMEOUT_INTERVAL_SYMBOLS_UPDATE,
-									   on_editor_buffer_symbols_update_timeout,
-									   plugin);
 	sdb_plugin->need_symbols_update = FALSE;
 }
 
@@ -901,7 +959,7 @@ do_add_new_files (SymbolDBPlugin *sdb_plugin, const GPtrArray *sources_array,
 			continue;
 		
 		gfile_info = g_file_query_info (gfile, 
-										"*", 
+										"standard::content-type", 
 										G_FILE_QUERY_INFO_NONE,
 										NULL,
 										NULL);
@@ -953,7 +1011,7 @@ do_add_new_files (SymbolDBPlugin *sdb_plugin, const GPtrArray *sources_array,
 		/* ok, we've just tested that the local_filename does exist.
 		 * We can safely add it to the array.
 		 */
-		g_ptr_array_add (languages_array, g_strdup (lang));					
+		g_ptr_array_add (languages_array, g_strdup (lang));
 		g_ptr_array_add (to_scan_array, g_strdup (local_filename));
 		g_object_unref (gfile);
 		g_object_unref (gfile_info);		
@@ -964,7 +1022,7 @@ do_add_new_files (SymbolDBPlugin *sdb_plugin, const GPtrArray *sources_array,
 	 */
 	if (to_scan_array->len > 0)
 	{		
-		gint proc_id = 	symbol_db_engine_add_new_files (sdb_plugin->sdbe_project, 
+		gint proc_id = 	symbol_db_engine_add_new_files_full (sdb_plugin->sdbe_project, 
 					sdb_plugin->project_opened, to_scan_array, languages_array, 
 														   TRUE);
 		
@@ -1068,13 +1126,14 @@ on_system_scan_package_start (SymbolDBEngine *dbe, guint num_files,
 	sdb_plugin->files_count_system_done = 0;
 	sdb_plugin->files_count_system += num_files;	
 	
-	DEBUG_PRINT ("********************* START [%s] with n %d files ", package, num_files);
-	
+		
 	/* show the global bar */
 	gtk_widget_show (sdb_plugin->progress_bar_system);
 	if (sdb_plugin->current_scanned_package != NULL)
 		g_free (sdb_plugin->current_scanned_package);
 	sdb_plugin->current_scanned_package = g_strdup (package);
+	
+	DEBUG_PRINT ("********************* START [%s] with n %d files ", package, num_files);
 }
 
 static void
@@ -1459,9 +1518,9 @@ static gboolean
 do_update_project_symbols (SymbolDBPlugin *sdb_plugin, const gchar *root_dir)
 {
 	gint proc_id;
-	/* Update the symbols */
+	/* Update the symbols */	
 	proc_id = symbol_db_engine_update_project_symbols (sdb_plugin->sdbe_project, 
-														 root_dir);
+														 root_dir, FALSE);
 	if (proc_id > 0)
 	{
 		sdb_plugin->is_project_updating = TRUE;		
@@ -1993,8 +2052,7 @@ on_scan_end_manager (SymbolDBEngine *dbe, gint process_id,
 	gint task_registered;
 	
 	task_registered = GPOINTER_TO_INT (g_tree_lookup (sdb_plugin->proc_id_tree, 
-													   GINT_TO_POINTER (process_id)		
-													  ));
+									 GINT_TO_POINTER (process_id)));
 	/* hey, we haven't find anything */
 	if (task_registered <= 0)
 	{
@@ -2079,7 +2137,7 @@ on_scan_end_manager (SymbolDBEngine *dbe, gint process_id,
  	 * then activate the display of local view
  	 */
 	enable_view_signals (sdb_plugin, TRUE, FALSE);
-	
+
 	if (sdb_plugin->is_offline_scanning == FALSE &&
 		 sdb_plugin->is_project_importing == FALSE &&
 		 sdb_plugin->is_project_updating == FALSE &&
@@ -2087,7 +2145,6 @@ on_scan_end_manager (SymbolDBEngine *dbe, gint process_id,
 	{
 		sdb_plugin->files_count_project_done = 0;
 		sdb_plugin->files_count_project = 0;
-		
 		clear_project_progress_bar (dbe, sdb_plugin);		
 	}	
 }
@@ -2152,8 +2209,11 @@ symbol_db_activate (AnjutaPlugin *plugin)
 	sdb_plugin->prefs = anjuta_shell_get_preferences (plugin->shell, NULL);
 	sdb_plugin->project_opened = NULL;
 
-	ctags_path = anjuta_preferences_get (sdb_plugin->prefs, CTAGS_PREFS_KEY); 
+	ctags_path = NULL;
 
+	/* leaving here this code. Maybe in future ctags-devs will include our patches
+	 * upstream and this can be useful again.
+	 */
 	if (ctags_path == NULL) 
 	{
 		DEBUG_PRINT ("ctags is not in preferences. Trying a default one %s", 
@@ -2178,6 +2238,7 @@ symbol_db_activate (AnjutaPlugin *plugin)
 	 */
 	sdb_plugin->buffer_update_files = g_ptr_array_new ();
 	sdb_plugin->buffer_update_ids = g_ptr_array_new ();
+	sdb_plugin->buffer_update_semaphore = FALSE;
 	
 	sdb_plugin->is_offline_scanning = FALSE;
 	sdb_plugin->is_project_importing = FALSE;
