@@ -41,9 +41,10 @@
 #define ANJUTA_DB_FILE	".anjuta_sym_db"
 
 /* if tables.sql changes or general db structure changes modify also the value here */
-#define SYMBOL_DB_VERSION	"1.0"
+#define SYMBOL_DB_VERSION	"228"
 
-#define TABLES_SQL	PACKAGE_DATA_DIR"/tables.sql"
+#define TABLES_SQL			PACKAGE_DATA_DIR"/tables.sql"
+#define TABLES_SQL_1_TO_228	PACKAGE_DATA_DIR"/tables-from-1-to-228.sql"
 
 #define CTAGS_MARKER	"#_#\n"
 
@@ -81,12 +82,13 @@
 #else
 #define MP_LEND_OBJ_STR(sdb_priv, OUT_gvalue) \
 		OUT_gvalue = (GValue*)g_queue_pop_head(sdb_priv->mem_pool_string); 
+		/*DEBUG_PRINT ("lend str %p, qlength %d [-]", OUT_gvalue, g_queue_get_length (sdb_priv->mem_pool_string));*/
 
 #define MP_RETURN_OBJ_STR(sdb_priv, gvalue) \
 	g_value_set_static_string (gvalue, DUMMY_VOID_STRING); \
 	g_queue_push_head(sdb_priv->mem_pool_string, gvalue); 
-
-
+	/*DEBUG_PRINT ("return str %p, qlength %d [+]", gvalue, g_queue_get_length (sdb_priv->mem_pool_string));*/
+	
 
 #define MP_LEND_OBJ_INT(sdb_priv, OUT_gvalue) \
 		OUT_gvalue = (GValue*)g_queue_pop_head(sdb_priv->mem_pool_int); 
@@ -259,6 +261,7 @@ typedef enum {
 /* the SymbolDBEngine Private structure */
 struct _SymbolDBEnginePriv
 {
+	gchar *anjuta_db_file;
 	gchar *ctags_path;
 	
 	GdaConnection *db_connection;
