@@ -81,29 +81,29 @@ static const struct
 	char *help;
 	ATPFlags flag;
 } variable_list[] = {
- {"project_root_uri", "project root URI", ATP_DEFAULT_VARIABLE},
- {"project_root_directory", "project root path", ATP_DIRECTORY_VARIABLE },
- {"file_manager_current_uri", "selected file manager URI", ATP_DEFAULT_VARIABLE },
- {"file_manager_current_directory", "selected file manager directory", ATP_DIRECTORY_VARIABLE },
- {"file_manager_current_full_filename", "selected file manager full file name", ATP_FILE_VARIABLE },
- {"file_manager_current_full_filename_without_ext", "selected file manager full file name without extension", ATP_FILE_VARIABLE },
- {"file_manager_current_filename", "selected file manager file name", ATP_FILE_VARIABLE },
- {"file_manager_current_filename_without_ext", "selected file manager file name without extension", ATP_FILE_VARIABLE },
- {"file_manager_current_extension", "selected file manager file extension", ATP_DEFAULT_VARIABLE },
- {"project_manager_current_uri", "selected project manager URI", ATP_DEFAULT_VARIABLE },
- {"project_manager_current_directory", "selected project manager directory", ATP_DIRECTORY_VARIABLE },
- {"project_manager_current_full_filename", "selected project manager full file name", ATP_FILE_VARIABLE },
- {"project_manager_current_full_filename_without_ext", "selected project manager full file name without extension", ATP_FILE_VARIABLE },
- {"project_manager_current_filename", "selected project manager file name", ATP_FILE_VARIABLE },
- {"project_manager_current_filename_without_ext", "selected project manager file name without extension", ATP_FILE_VARIABLE },
- {"project_manager_current_extension", "selected project manager file extension", ATP_DEFAULT_VARIABLE },
- {"editor_current_filename", "current edited file name", ATP_FILE_VARIABLE },
- {"editor_current_filename_without_ext", "current edited file name without extension", ATP_FILE_VARIABLE },
- {"editor_current_directory", "current edited file directory", ATP_DIRECTORY_VARIABLE },
- {"editor_current_selection", "current selection in editor", ATP_FILE_VARIABLE },
- {"editor_current_word", "current word in editor", ATP_FILE_VARIABLE },
- {"editor_current_line", "current line in editor", ATP_FILE_VARIABLE },
- {"ask_user_string", "ask the user to get additional parameters", ATP_INTERACTIVE_VARIABLE }
+ {IANJUTA_PROJECT_MANAGER_PROJECT_ROOT_URI, N_("Project root URI"), ATP_DEFAULT_VARIABLE},
+ {"project_root_directory", N_("Project root path"), ATP_DIRECTORY_VARIABLE },
+ {"file_manager_current_uri", N_("Selected URI in the file manager plugin"), ATP_DEFAULT_VARIABLE },
+ {"file_manager_current_directory", N_("Selected directory in the file manager plugin"), ATP_DIRECTORY_VARIABLE },
+ {"file_manager_current_full_filename", N_("Selected full file name in the file manager plugin"), ATP_FILE_VARIABLE },
+ {"file_manager_current_full_filename_without_ext", N_("Selected full file name without extension in the file manager plugin"), ATP_FILE_VARIABLE },
+ {"file_manager_current_filename", N_("Selected file name in the file manager plugin"), ATP_FILE_VARIABLE },
+ {"file_manager_current_filename_without_ext", N_("Selected file name without extension in the file manager plugin"), ATP_FILE_VARIABLE },
+ {"file_manager_current_extension", N_("Selected file's extension in the file manager plugin"), ATP_DEFAULT_VARIABLE },
+ {"project_manager_current_uri", N_("Selected URI in the project manager plugin"), ATP_DEFAULT_VARIABLE },
+ {"project_manager_current_directory", N_("Selected directory in the project manager plugin"), ATP_DIRECTORY_VARIABLE },
+ {"project_manager_current_full_filename", N_("Selected full file name in the project manager plugin"), ATP_FILE_VARIABLE },
+ {"project_manager_current_full_filename_without_ext", N_("Selected full file name without extension in the project manager plugin"), ATP_FILE_VARIABLE },
+ {"project_manager_current_filename", N_("Selected file name in the project manager plugin"), ATP_FILE_VARIABLE },
+ {"project_manager_current_filename_without_ext", N_("Selected file name without extension in the project manager plugin"), ATP_FILE_VARIABLE },
+ {"project_manager_current_extension", N_("Selected file extension in the project manager plugin"), ATP_DEFAULT_VARIABLE },
+ {"editor_current_filename", N_("Currently edited file name"), ATP_FILE_VARIABLE },
+ {"editor_current_filename_without_ext", N_("Currently edited file name without extension"), ATP_FILE_VARIABLE },
+ {"editor_current_directory", N_("Currently edited file directory"), ATP_DIRECTORY_VARIABLE },
+ {"editor_current_selection", N_("Currently selection in editor"), ATP_FILE_VARIABLE },
+ {"editor_current_word", N_("Current word in editor"), ATP_FILE_VARIABLE },
+ {"editor_current_line", N_("Current line in editor"), ATP_FILE_VARIABLE },
+ {"ask_user_string", N_("Ask the user to get additional parameters"), ATP_INTERACTIVE_VARIABLE }
 };
 
 /* Helper functions
